@@ -1,8 +1,7 @@
 from ..imports import *
 from ..factories import connect_signals, apply_properties
-from ..factories.table_factory_headers_config import createTable
-from abstract_utilities import make_list
-
+from ..factories.table_factory_headers_config import set_table_data,_normalize_headers_and_data
+from abstract_utilities import make_list,is_number
 # ---- Qt5/Qt6 compat for QSizePolicy ----
 try:
     # Qt6
@@ -156,7 +155,16 @@ def getAddWidget(parent=None, layout=None, widget=None):
         addWidget(layout, widget)
     elif parent is not None:
         addWidget(parent, widget)
-
+def getHorizontalHeader(table=None,col=None,arg=None):
+    if table and arg:
+        if is_number(col):
+            if arg not in [None,True,False]:
+                attr = getattr(QHeaderView.ResizeMode, arg)
+                table.horizontalHeader().setSectionResizeMode(col,attr)
+            elif isinstance(arg,'QHeaderView.ResizeMode'):
+                table.horizontalHeader().setSectionResizeMode(col, arg)
+        elif isinstance(arg,bool):
+            table.horizontalHeader().setStretchLastSection(arg)
 # ---------------- row + builders ----------------
 
 def make_input_row(
@@ -219,7 +227,71 @@ def createCombo(parent, layout=None, widget_cls=None, items=None, label=None, at
     getConnect(connect, box)
     getAddWidget(parent=parent, layout=layout, widget=box)
     return box
+def createTable(
+    parent: QWidget,
+    layout: Optional[QVBoxLayout] = None,
+    label: Optional[str] = None,
+    args=None,
+    attr_name: Optional[str] = None,
+    headers: Optional[Sequence[str]] = None,
+    headers_config: Optional[Dict[str, Dict[str, Any]]] = None,
+    data: Optional[Iterable] = None,
+    selection_behavior: QAbstractItemView.SelectionBehavior = QAbstractItemView.SelectionBehavior.SelectRows,
+    selection_mode: QAbstractItemView.SelectionMode = QAbstractItemView.SelectionMode.SingleSelection,
+    edit_triggers: QAbstractItemView.EditTrigger = (
+        QAbstractItemView.EditTrigger.DoubleClicked | QAbstractItemView.EditTrigger.EditKeyPressed
+    ),
+    resize_mode_default: QHeaderView.ResizeMode = QHeaderView.ResizeMode.ResizeToContents,
+    stretch_last_section: bool = False,
+    props: Optional[Dict[str, Any]] = None,
+    connect: Optional[Any] = None,
+    connect_signals_names: Optional[List[str]] = None,
+    prepend_widget: bool = True,
+    **kwargs
+) -> QTableWidget:
 
+
+    layout = layout or QVBoxLayout(parent)
+    if label:
+        layout.addWidget(QLabel(label))
+    args = make_list(args) or []
+    table = QTableWidget(*args)
+    table.setSelectionBehavior(selection_behavior)
+    table.setSelectionMode(selection_mode)
+    table.setEditTriggers(edit_triggers)
+    table.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
+
+    # If headers_config provided, derive headers order from dict keys (ordered)
+    if headers_config:
+        headers = list(headers_config.keys())
+
+    # Seed data
+    if data is not None or headers is not None:
+        set_table_data(table, data or [], headers=headers)
+
+    # Column-level header config
+ 
+    if headers_config:
+        for col, name in enumerate(headers or []):
+            cfg = headers_config.get(name, {})
+            # horizontalHeader resize mode
+            mode = cfg.get("horizontalHeader",resize_mode_default)
+            getHorizontalHeader(table,col,mode)
+            # explicit width
+            colWidth = cfg.get("ColumnWidth")
+            if is_number(colWidth):
+                table.setColumnWidth(col, int(colWidth))
+        # stretch last if asked or keep caller default
+        getHorizontalHeader(tabl=table,arg=stretch_last_section)
+    else:
+        # simple default if no per-column config
+        table.horizontalHeader().setSectionResizeMode(resize_mode_default)
+        table.horizontalHeader().setStretchLastSection(stretch_last_section)
+    apply_properties(table,kwargs)
+    getParentAttr(parent, table, attr_name=attr_name, label=label)
+    getConnect(connect, chk)
+    getAddWidget(parent=parent, layout=layout, widget=table)
+    return table
 def createButton(parent, layout=None, widget_cls=None, label=None, attr_name=None, connect=None, **kwargs):
     widget_cls = widget_cls or QPushButton
     text = label or "button"
